import chalk from "chalk";
import webpack from "webpack";
import ora from "ora";
import ProgressBar from "./ProgressBar";

const default_pattern = chalk.bold('[')+':percent:%'+chalk.bold('] ')+chalk.cyan.bold(':text:');

export interface Options {
  stream?: NodeJS.WriteStream;
  pattern?: string;
  pattern_no_stderr?: string;
  update_render?: boolean;
  clear?: boolean;
  clear_on_update?: boolean;
  stderr_check?: boolean;
  text?: string;
}

class ProgressOraPlugin {
  private pattern: string;
  private pattern_no_stderr: string;
  private update_render: boolean;
  private clear: boolean;
  private clear_on_update: undefined | boolean;
  private pb: any;
  private enable = true;
  private stderr_check: boolean;
  private text: string;
  private spinner: ora.Ora | null;
  private stream: any;

  constructor(options: Options = {}) {
    this.pattern = options.pattern || default_pattern
    this.pattern_no_stderr = options.pattern_no_stderr || chalk.bold('█');
    this.update_render = options.update_render || false;
    this.clear = options.clear || false;
    this.clear_on_update = options.clear_on_update || false;
    this.pb = new ProgressBar('Generate...', 20, options.pattern_no_stderr);
    this.stderr_check = false;
    this.stream = options.stream || process.stderr;
    let enabled = this.stream && this.stream.isTTY;
    if (!enabled) {
      if (!options.stderr_check) {
        this.enable = false
      }
      this.stderr_check = true;
    }

    if(options.clear && !this.stderr_check) {
      this.stream.write('\x1Bc');
    } else {
      this.stream.write('\n');
    }

    this.text = this.pattern.replace(/\:percent\:/, '0').replace(/\:text\:/, 'build start');

    this.clear_on_update = options.clear_on_update
    this.spinner = !this.stderr_check ? ora(options).start() : null;
  }

  apply(compiler: webpack.Compiler) {
    if(!this.enable) return;

    let isRunning = false;
    let startTime = 0;

    return new webpack.ProgressPlugin((percent: number, msg: string) => {
      let newPercent = Math.ceil(percent * 100);

      if (!this.stderr_check) {
        if(this.clear_on_update) {
          this.stream.write('\x1Bc');
        }
        if(this.spinner && this.pattern) this.spinner.text = this.pattern.replace(/\:percent\:/, newPercent.toString()).replace(/\:text\:/, msg)
        if(this.spinner && this.update_render) {
          this.spinner.render();
        }
      } else {
        this.pb.render({
          percent: (newPercent / 100).toFixed(4),
          completed: newPercent,
          total: 100,
        })
      }

      if (!isRunning) {
        isRunning = true;
        startTime = (new Date()).getTime();
      } else if (percent >= 1) {
        let now = (new Date()).getTime();
        let buildTime = (now - startTime) / 1000 + 's';

        if(this.stderr_check) {
          this.stream.write('\n' +chalk.green('✓') + chalk.green.bold(' Build completed in ' + buildTime + '\n\n'));
        } else {
          if(this.spinner) this.spinner.stop();

          if(this.clear) {
            this.stream.write('\x1Bc');
          }
          if(this.spinner) this.spinner.succeed(chalk.green.bold('Build completed in ' + buildTime + '\n\n'))
        }
        isRunning = false;
      }
    });
  }
}


export default ProgressOraPlugin;

export { ProgressOraPlugin }
